/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DiAMAGE. */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <signal.h>

/* TODO: when libc and kernel are ready for a non-provisoric shell, rewrite this
 * shell. */

static void exec_cmd(char **argv)
{
	int r;

	if (!strcmp(argv[0], "exit"))
		exit(EXIT_SUCCESS);

	r = fork();
	if (r == -1)
		perror("fork");
	else if (r == 0) {
		execve(argv[0], argv, environ);
		perror("execve");
		exit(EXIT_FAILURE);
	} else
		wait(0);
}

/* returns argv pointer */
static char **setup_arg(char *cmd)
{
	int i, spaces = 0, cur = 0;
	char **ret;

	/* calculate needed pointers */
	for (i = 0; cmd[i]; i++)
		if (cmd[i] == ' ')
			spaces++;

	spaces++;
	ret = calloc(spaces + 1, sizeof(char *));

	/* and now setup pointers */
	ret[cur++] = cmd;
	for (i = 0; cmd[i]; i++)
		if (cmd[i] == ' ') {
			cmd[i++] = 0;
			ret[cur++] = &cmd[i];
		}

	return ret;
}

int main(int argc, char **argv)
{
	char buf[128];
	int r;
	int i;
	struct sigaction sa;

	(void) argc;
	(void) argv;

	char **cmdargv;

	sa.sa_flags = 0;
	sa.sa_handler = SIG_IGN;
	sigaction(SIGINT, &sa, 0);

	while (1) {
		write(STDOUT_FILENO, "# ", 2);

		r = read(STDIN_FILENO, buf, sizeof(buf));
		if (r <= 0)
			break;

		for (i = 0;
		     i < (int) sizeof(buf) && buf[i] != ';' && buf[i] != '\n';
		     i++) ;

		if (i == sizeof(buf))
			/* buffer overflow... */
			buf[i - 1] = 0;
		else
			buf[i] = 0;

		cmdargv = setup_arg(buf);
		exec_cmd(cmdargv);
		free(cmdargv);

		if (r > i + 1)
			memmove(buf, &buf[i+1], r - i);
	}

	if (r == -1) {
		perror("read");
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}
