/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once

#include <sys/functions.h>
#include <sys/types.h>
#include <signal.h>

#define EXIT_FAILURE	1
#define EXIT_SUCCESS	0

extern int atexit(void (*func)(void));
extern __noreturn exit(int status);
#define _Exit exit

extern void *malloc(size_t size);
extern void *calloc(size_t nelem, size_t elsize);
extern void free(void *ptr);
extern void *realloc(void *ptr, size_t size);

#define alloca(s)	__builtin_alloca(s)

static inline __noreturn abort(void)
{
	raise(SIGABRT);
	for (;;) ;	/* to make compiler believe we do not return (altough
			 * raise(SIGABRT) does not return but noone knows
			 * that) */
}

extern void qsort(void *base, size_t nel, size_t width,
		  int (*compar)(const void *, const void *));

extern long strtol(const char *str, char **endptr, int base);
static inline int atoi(const char *str)
{
	return strtol(str, 0, 10);
}
