/* Copyright (c) 2009, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once

#include <errno.h>
#include <sys/types.h>
#include <sys/kernel/syscall.h>
#include <sys/kernel/page.h>
#include <sys/functions.h>

#define MAP_FAILED	0

static inline void *mmap(void *addr, size_t len, int prot, int flags,
			 int fildes, off_t off)
{
	register int r asm("r0");
	register int a asm("r0") = (int) addr;
	register int l asm("r1") = len;
	register int p asm("r2") = prot;
	register int fl asm("r3") = flags;
	register int fd asm("r4") = fildes;
	register int o asm("r5") = off;
	asm volatile ("swi %[sc]" :
		      "=r" (r) :
		      "r" (a),
		      "r" (l),
		      "r" (p),
		      "r" (fl),
		      "r" (fd),
		      "r" (o),
		      [sc] "i" (SC_MMAP));
	if (r > 0 && r <= ERRNO_MAX) {
		errno = r;
		return MAP_FAILED;
	}
	return (void *) r;
}

__syscalli2(int, munmap, SC_MUNMAP, void *, size_t)
