/* Copyright (c) 2008-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#ifndef KERNEL
#include "malloc.h"
#endif

#ifdef KERNEL
#define free_page(x)	page_free(x, 1)
#else
#define free_page(x)	munmap(x, PAGE_SIZE)
#endif

void free(void *ptr)
{
	int blkcnt, blknr;
	struct mpg *mpg, *next, *cur;

	if (!ptr)
		/* malloc(0) returns 0, so this is valid */
		return;

	ptr -= 4;

	mpg = find_mpg(ptr);

	/* get block number in this mpg */
	blknr = ((unsigned long) ptr - (unsigned long) mpg) / BLOCKSIZE;

	/* get block count */
	blkcnt = *(int *) ptr;

	/* mark blocks as free */
	do_free(mpg, blknr, blknr + blkcnt);

	/* check whether we just freed the last block in this mpg */
	if (mpg->freeblocks == INITFREE) {
		/* if yes, unlink and free this mpg */

		cur = __malloc_pages;
		next = mpg->next;

		free_page(mpg);

		if (__malloc_pages == mpg) {
			__malloc_pages = next;
			return;
		}
		do {
			if (cur->next == mpg) {
				cur->next = next;
				return;
			}
		} while ((cur = cur->next) != 0);

		/* if we are here, the mpg was not linked. */
	}
}
