/* Copyright (c) 2008-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#ifndef KERNEL
#include "malloc.h"
#endif

#ifndef KERNEL
struct mpg *__malloc_pages;
#endif

#ifdef KERNEL
#define get_page()	page_alloc(0, 1, PROT_READ | PROT_WRITE)
#else
#define get_page()	mmap(0, PAGE_SIZE, PROT_READ | PROT_WRITE,	\
			     MAP_PRIVATE | MAP_ANON, 0, 0)
#endif

void *malloc(size_t size)
{
	int i, j;
	int blk;
	struct mpg *mpg;
	void *rv;

	/* TODO: support sizes bigger than DATASIZE */

	if (!size)
		/* save time & space */
		return 0;

	size += 4;	/* needing an int to store length of allocation */

	blk = blocks(size, BLOCKSIZE);

	/* for each entry in the linked list scan for enough free blocks */
scan:
	for (mpg = __malloc_pages; mpg; mpg = mpg->next) {
		if (mpg->freeblocks < blk)
			/* too few free blocks */
			continue;

		for (i = FIRSTDATABLOCK; i + blk < BPE; i++) {
			if (mpg->bitmap[i / 32] & 1 << (i % 32))
				/* block used */
				continue;

			/* check whether there are enough continious free
			 * blocks */
			for (j = i + 1; j < i + blk; j++)
				if (mpg->bitmap[j / 32] & 1 << (j % 32))
					break;

			if (j < i + blk) {
				/* not enough free blocks */
				i = (j - 1);
				continue;
			}

			break;
		}

		if (i + blk < BPE)
			/* we found something */
			break;
	}

	if (!mpg) {
		/* all dms we have are too full. create a new one */

		if (!__malloc_pages) {
			/* not even one mpg entry is allocated */
			__malloc_pages = get_page();
			__malloc_pages->next = 0;
			__malloc_pages->freeblocks = INITFREE;
			for (i = 0; i < BITMAPSIZE; i++)
				__malloc_pages->bitmap[i] = 0;
			goto scan;
		}

		/* find last mpg in queue */
		for (mpg = __malloc_pages; mpg->next; mpg = mpg->next) ;

		/* create entry */
		mpg->next = get_page();
		mpg->next->next = 0;
		mpg->next->freeblocks = INITFREE;
		for (i = 0; i < BITMAPSIZE; i++)
			mpg->next->bitmap[i] = 0;
		goto scan;
	}

	/* mark blocks as used */
	do_alloc(mpg, i, blk + i);

	/* get address of freshly allocated allocation */
	rv = (void *) ((unsigned long) mpg + i * BLOCKSIZE);

	/* save number of blocks */
	*(int *) rv = blk;

	/* return */
	return rv + 4;
}
