/* Copyright (c) 2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#ifndef KERNEL
#include <string.h>
#endif

void __optimize *memcpy(void *s1, const void *s2, size_t n)
{
	char *d = s1;
	const char *s = s2;
	int *r;
	const int *q;

	if (n < 16)
		/* too much overhead for using wordwise memcpy */
		goto bytewise;

	if (((int) s1 & 3) != ((int) s2 & 3))
		/* pointers cannot be both aligned to word boundaries */
		goto bytewise;

	/* align on word boundary */
	while (((int) d & 3) != 0) {
		*d++ = *s++;
		n--;
	}

	/* wordwise copying till no whole word is left */
	r = (int *) d;
	q = (const int *) s;
	while (n >= 4) {
		*r++ = *q++;
		n -= 4;
	}

	/* do bytewise what is missing */
	d = (char *) r;
	s = (const char *) q;
bytewise:
	while (n--)
		*d++ = *s++;

	return s1;
}
