/* Copyright (c) 2009, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once

/* program status register flags */
#define PSR_M_usr	0x10
#define PSR_M_fiq	0x11
#define PSR_M_irq	0x12
#define PSR_M_svc	0x13
#define PSR_M_abt	0x17
#define PSR_M_und	0x1b
#define PSR_M_sys	0x1f
#define PSR_F		(1 << 6)
#define PSR_I		(1 << 7)

/* initalize stack pointers and zero out .bss */
#define init_bss					\
	msr	CPSR_c, (PSR_M_irq | PSR_F | PSR_I);	\
	ldr	sp,	=(RAM_START + RAM_SIZE);	\
	msr	CPSR_c,	(PSR_M_svc | PSR_F | PSR_I);	\
	ldr	sp,	=(RAM_START + RAM_SIZE);	\
	msr	CPSR_c,	(PSR_M_und | PSR_F | PSR_I);	\
	ldr	sp,	=(RAM_START + RAM_SIZE);	\
	msr	CPSR_c,	(PSR_M_abt | PSR_F | PSR_I);	\
	ldr	sp,	=(RAM_START + RAM_SIZE);	\
	msr	CPSR_c, (PSR_M_sys | PSR_F | PSR_I);	\
	ldr	sp,	=(RAM_START + RAM_SIZE);	\
	mov	r0,	#0;				\
	ldr	r1, 	=RAM_START;			\
	ldr	r2,	=__bss_end__;			\
2:	cmp	r1,	r2;				\
	strlo	r0,	[r1],	#4;			\
	blo	2b

#ifndef __ASSEMBLER__

#include "compiler.h"

/* idle function for scheduler.
 * Could also be used to freeze the system as it is powersaving and does not
 * return */
extern __noreturn idle(void);

/* lr function for signals.
 * This calls the syscall to restore the signal context */
extern __noreturn sig_lrfunc(void);

#endif
