/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#include "bio.h"
#include "dev.h"
#include "devices.h"
#include "errno.h"
#include "lib.h"

static int devnull_read(struct filedes *fd, void *buf, unsigned int count)
{
	(void) fd;
	(void) buf;
	(void) count;

	return 0;
}

static int devnull_write(struct filedes *fd, const void *buf,
			 unsigned int count)
{
	(void) fd;
	(void) buf;

	return count;
}

static const struct io_handle devnull_ioh = {
	.open 		= def_open,
	.close		= def_close,
	.clone_fd	= def_clone_fd,
	.read 		= devnull_read,
	.write		= devnull_write,
	.poll		= def_poll
};

struct devfs_ent {
	struct devfs_ent *next;
	const char *name;
	const struct io_handle *ioh;
	void *ud;
};
static struct devfs_ent *devfs_ents;

void devfs_add(const char *name, const struct io_handle *ioh, void *ud)
{
	struct devfs_ent *d;
	struct devfs_ent *new = malloc(sizeof(struct devfs_ent));
	if (!devfs_ents)
		devfs_ents = new;
	else {
		for (d = devfs_ents; d->next; d = d->next) ;
		d->next = new;
	}

	new->next = 0;
	new->name = name;
	new->ioh  = ioh;
	new->ud   = ud;
}

static int devfs_mount(struct mountpoint *mntp)
{
	(void) mntp;

	devfs_add("null", &devnull_ioh, 0);
	/* initalized before devfs. thus can't call devfs_add itself on init */
	/* TODO: make this nicer (dmesg, and no direct uart calls) */
	uart_late_init();

	return 0;
}

static int devfs_open(struct filedes *fd, const char *path)
{
	struct devfs_ent *d;

	if (!strcmp(path, "console"))
		path = SYSTEM_CONSOLE;

	for (d = devfs_ents; d; d = d->next)
		if (!strcmp(d->name, path)) {
			fd->ioh = d->ioh;
			fd->ud  = d->ud;
			return fd->ioh->open(fd, path);
		}

	return -ENOENT;
}

const struct io_handle devfs_ioh = {
	.mount = devfs_mount,
	.open  = devfs_open
	/* other functions are handled by device node io_handles */
};
