/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#ifndef ERRNO_NUMBERS_DEFINED

#define ERRNO_NUMBERS_DEFINED

#define EAGAIN	1	/* Resource temporarily unavailable */
#define EBADF	2	/* Bad file descriptor */
#define ECHILD	3	/* No child processes */
#define EFAULT	4	/* Bad address */
#define EILSEQ	5	/* Illegal byte sequence */
#define EINTR	6	/* Interrupted function */
#define EINVAL	7	/* Invalid argument */
#define EIO	8	/* I/O error */
#define ELOOP	9	/* Too many levels of links */
#define ENOENT	10	/* No such file or directory */
#define ENOEXEC	11	/* Executable file format error */
#define ENOMEM	12	/* Not enough space */
#define ENOTDIR	13	/* Not a directory */
#define ENOTSUP	14	/* Not supported */
#define EPERM	15	/* Operation not permitted */
#define EROFS	16	/* Read-only file system */
#define ESRCH	17	/* No such process */

/* maximum possible errno number */
#define ERRNO_MAX	ESRCH

#endif

/* should not be used in the kernel. this table is too big */
#if !defined(KERNEL) && defined(DEFINE_ERRNO_STRTABLE)
static const char *__errno_str[ERRNO_MAX] = {
	[EAGAIN-1]	= "Resource temporarily unavailable",
	[EBADF-1]	= "Bad file descriptor",
	[ECHILD-1]	= "No child processes",
	[EFAULT-1]	= "Bad address",
	[EILSEQ-1]	= "Illegal byte sequence",
	[EINTR-1]	= "Interrupted function",
	[EINVAL-1]	= "Invalid argument",
	[EIO-1]		= "I/O error",
	[ELOOP-1]	= "Too many levels of links",
	[ENOENT-1]	= "No such file or directory",
	[ENOEXEC-1]	= "Executable file format error",
	[ENOMEM-1]	= "Not enough space",
	[ENOTDIR-1]	= "Not a directory",
	[ENOTSUP-1]	= "Not supported",
	[EPERM-1]	= "Operation not permitted",
	[EROFS-1]	= "Read-only file system",
	[ESRCH-1]	= "No such process"
};
#endif

#ifdef KERNEL

/* This shall be returned as errno number on functions which do not yet return.
 * This is the case if a function causes a task switch because it is blocking.
 * The syscall handler does not change the r0 register of the calling process,
 * because this would be fatal (because it would change the r0 register of
 * another process due to the task switch). */
#define DONT_RETURN	-(ERRNO_MAX + 1)

#endif
