/* Copyright (c) 2009, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

.text
.arm

#define isr_stub(cisr)										\
	/* store return address (lr_[mode]) to stack */						\
	stmfd	sp!,	{ lr };									\
	/* store user mode registers on stack */						\
	stmfd	sp,	{ r0, r1, r2, r3, r4, r5, r6, r7, r8, r9, r10, r11, r12, sp, lr }^;	\
	sub	sp,	sp,	#60;								\
	/* store user mode program status register on stack */					\
	mrs	r0,	SPSR;									\
	stmfd	sp!,	{ r0 };									\
	/* call handler */									\
	mov	r0,	sp;									\
	bl	cisr;										\
	/* rewrite prorgram status register (to SPSR, as the last ldmfd writes SPSR to CPSR) */	\
	ldmfd	sp!,	{ r0 };									\
	msr	SPSR_cxsf, r0;									\
	/* rewrite registers */									\
	ldmfd	sp,	{ r0, r1, r2, r3, r4, r5, r6, r7, r8, r9, r10, r11, r12, sp, lr }^;	\
	add	sp,	sp,	#60;								\
	ldmfd	sp!,	{ pc }^

.global isr_timer
isr_timer:
	sub	lr,	lr,	#4
	isr_stub(sched_timer)

.global isr_uart
isr_uart:
	sub	lr,	lr,	#4
	isr_stub(uart_irq)

.global isr_syscall
isr_syscall:
	isr_stub(syscall)

.global isr_undefined
isr_undefined:
	isr_stub(exc_undefined)

.global isr_prefetch
isr_prefetch:
	sub	lr,	lr,	#4
	isr_stub(isr_prefetch)

.global isr_data
isr_data:
	sub	lr,	lr,	#8
	isr_stub(exc_data)

/* ISR for unexpected exceptions which just can't happen */
.global unexpected
unexpected:
	ldr	r0,	=unexpected_str
	b	kpanic

.section .rodata
unexpected_str: .asciz "unexpected exception happened"
