/* Copyright (c) 2008-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#include "bio.h"
#include "devices.h"
#include "errno.h"
#include "lib.h"
#include "page.h"

/* needed due to the lib includings */
#define size_t unsigned int

/* stdio */

#include "../lib/stdio/sprintf.c"
#include "../lib/stdio/vsprintf.c"

/* printf' buffer size */
#define PRINTF_BUF	128

/* TODO: add dmesg stuff and stop the need of uart (first into dmesg buffer
 * only, then init console device, then out to console) */

int vprintf(const char *fmt, va_list ap)
{
	char buf[PRINTF_BUF];
	int rv, i;

	rv = vsprintf(buf, fmt, ap);

	for (i = 0; i < rv; i++)
		uart_putchar(buf[i]);

	return rv;
}

int printf(const char *fmt, ...)
{
	int rv;
	va_list ap;

	va_start(ap, fmt);
	rv = vprintf(fmt, ap);
	va_end(ap);

	return rv;
}

int puts(const char *s)
{
	while (*s)
		uart_putchar(*s++);
	uart_putchar('\n');
	return 0;
}

/* string */

#include "../lib/string/memcpy.c"
#include "../lib/string/memmove.c"
#include "../lib/string/memset.c"
#include "../lib/string/strchr.c"
#include "../lib/string/strcmp.c"
#include "../lib/string/strcpy.c"
#include "../lib/string/strdup.c"
#include "../lib/string/strlen.c"
#include "../lib/string/strncmp.c"

/* stdlib */

#include "../lib/stdlib/malloc.h"
#include "../lib/stdlib/malloc.c"
#include "../lib/stdlib/free.c"
#include "../lib/stdlib/realloc.c"
