/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once

#include <stdarg.h>

#include "compiler.h"

/* stdio */
extern int vsprintf(char *, const char *, va_list);
extern int sprintf(char *, const char *, ...);
extern int vprintf(const char *, va_list);
extern int printf(const char *, ...);
extern int puts(const char *);

/* string */
extern void *memcpy(void *, const void *, unsigned int);
extern void *memmove(void *, const void *, unsigned int);
extern void *memset(void *, int, unsigned int);
extern unsigned int strlen(const char *);
extern int strncmp(const char *, const char *, unsigned int);
extern int strcmp(const char *, const char *);
extern char *strchr(const char *, int);
extern char *strdup(const char *);
extern char *strcpy(char *, const char *);

/* stdlib */
extern void *malloc(unsigned int);
extern void free(void *);
extern void *realloc(void *ptr, unsigned int size);
#define alloca(s)	__builtin_alloca(s)

/* nowhere */

/* It is often smaller to have this function not inlined, but inlining this
 * function avoids the need of useless instructions in many cases.
 *
 * s is the size, b the blocksize.  This function returnes the number of blocks
 * needed to store size. */
#define blocks(s, b)	(((s) % (b) == 0) ? ((s) / (b)) : ((s) / (b) + 1))

#define min(a, b)	(((a) < (b)) ? (a) : (b))
#define max(a, b)	(((a) > (b)) ? (a) : (b))

#define square(x)	((x) * (x))

/* in fault.c, altough defined in lib.h.
 * halts the system */
extern __noreturn kpanic(const char *s, ...) __printf(1, 2);
