/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#include "bio.h"
#include "dev.h"
#include "devices.h"
#include "exec.h"
#include "lib.h"
#include "page.h"
#include "sched.h"
#include "pf/cpu.h"
#include "syscall.h"

extern void init(void);

void init(void)
{
	led_init();
	uart_init();
	page_init();
	sched_init();
}

void init2(void)
{
	int r;
	char *argv[2] = { (char *) (int) CONFIG_INIT_PROG, 0 };
	char *envp[1] = { 0 };

	/* TODO: add version  */
	printf("Armux 0.0\n");
	printf("Compiled on "__DATE__", "__TIME__" for "PLATFORM_NAME".\n");

	bio_init();

	/* create console file descriptor */
	/* TODO: use nicer way (fds using and dmesg) */
	bio_open(0, "/dev/console", O_WRONLY, 0);

#ifdef CONFIG_FFI
	ffi_init();
#endif
#ifdef CONFIG_LPCMCI
	mci_init();
#endif
#ifdef CONFIG_SWMCI
	mmc_init();
#endif

	/* mount root filesystem */
	r = bio_mount("/dev/" CONFIG_ROOTFS_DEV, "/", CONFIG_ROOTFS_TYPE);
	if (r < 0)
		kpanic("Unable to mount root filesystem, errno %i", r);

	led_indicate_kernelinit();

	register int rv asm("r0");
	register int p asm("r0") = (int) CONFIG_INIT_PROG;
	register int a asm("r1") = (int) argv;
	register int e asm("r2") = (int) envp;
	asm volatile ("swi %[s]" :
		      "=r" (rv) :
		      "r" (p),
		      "r" (a),
		      "r" (e),
		      [s] "i" (SC_EXECVE));

	/* execve normally does not return */
	kpanic("execution of " CONFIG_INIT_PROG " failed, errno %i", rv);
}
