/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once

#define PAGE_SIZE	4096

/* parameters for mmap. These are also saved for every allocated page */
#define PROT_NONE	0
#define PROT_READ	(1 << 12)
#define PROT_WRITE	(1 << 13)
#define PROT_EXEC	(1 << 14)
#define MAP_SHARED	0
#define MAP_PRIVATE	(1 << 0)
#define MAP_FIXED	(1 << 1)
#define MAP_ANON	(1 << 2)

#ifdef KERNEL

/* size of kernel stack used for task switches, syscall handlink, exception
 * handling and initalization.
 * Size is in pages.
 * To set stack size of processes look at sched.h, SCHED_PROC_SP. */
#define KERN_STACKSIZE	(CONFIG_KERN_STACKSIZE / PAGE_SIZE)

extern void page_init(void);
extern void *page_alloc(int tid, int n, int prot);
extern int page_free(const void *ptr, int n);
extern void page_free_all(int tid);
extern void *sys_mmap(int tid, void *addr, int len, int prot, int flags,
		      int fildes, int off);
extern int sys_munmap(void *addr, int len);
extern int is_in_ram(void *addr);
extern int page_get_total(void);
extern int page_get_pertid(int tid);

#ifdef CONFIG_PTRCHECK

extern int ptr_valid(void *ptr, int tid, int prot, int size);
extern int ptr_valid_string(char *ptr, int tid);
extern int ptr_valid_strarray(char **ptr, int tid);

#else

#define ptr_valid(a, b, c, d)		1
#define ptr_valid_string(a, b)		1
#define ptr_valid_strarray(a, b)	1

#endif

#endif
