/* Copyright (c) 2009, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once

#define PLATFORM_NAME	CONFIG_CPU_VARIANT

/* our PLL multiplicator */
#define PLL_MUL		4

/* crystal clock in Hz */
#define CRYSTAL_CLOCK	14745600

/* CPU clock in Hz */
#define CPU_CLOCK	(CRYSTAL_CLOCK * PLL_MUL)

/* MAM cycles
 * According to the data sheet, MAM timings may be set:
 *  System Clock [MHz]	MAM
 *    0 - 20		1
 *   20 - 40		2
 *   40 - 60		3
 *  so:
 *   60 - 80		4
 *   80 - 100		5
 *  100 - 120		6
 *  120 - 140		7
 *  So, to get the optimal MAM timing for the system clock, we use: */
#define MAM_CCLKS	(CPU_CLOCK / 20000000 + 1)

#define RAM_START	0x40000000

#ifdef CONFIG_CPU_VARIANT_LPC2104
# define RAM_SIZE	(16 << 10)
#endif
#ifdef CONFIG_CPU_VARIANT_LPC2105
# define RAM_SIZE	(32 << 10)
#endif
#ifdef CONFIG_CPU_VARIANT_LPC2106
# define RAM_SIZE	(64 << 10)
#endif

/* UART0 base */
#define UART0_BASE	0xe000c000

/* base of timer0 used for scheduling */
#define TIMER0_BASE	0xe0004000

/* VIC registers */
#define VICIntEnable	(* (volatile int  *)  0xfffff010)
#define VICcVctAddr	(* (volatile long *)  0xfffff030)
#define VICDefVectAddr	(* (volatile long *)  0xfffff034)
#define VICVectAddr(x)	(* (volatile long *) (0xfffff100 + x*4))
#define VICVectCntl(x)	(* (volatile int  *) (0xfffff200 + x*4))
#define VICVectCntl_E	(1 << 5)	/* interrupt enabled bit */

#define TIMER_IRQ	4	/* timer interrupt number */
#define UART_IRQ	6	/* uart0 interrupt number */

#define cpu_idle_asm						\
	ldr	r0,	=0xe01fc0c0;	/* PCON register */	\
	mov	r1,	#(1 << 0);	/* Idle mode */		\
	str	r1,	[r0];

#ifndef __ASSEMBLER__

#include "isr.h"

/* sets a function for an interrupt as ISR */
static inline void vic_set(int nr, void (*isr)(void))
{
	VICIntEnable |= (1 << nr);

	VICDefVectAddr = (long) unexpected;

	VICVectAddr(nr) = (long) isr;
	VICVectCntl(nr) = VICVectCntl_E | nr;
}

/* resets VIC. Should be called at the end of every ISR */
static inline void vic_reset(void)
{
	VICcVctAddr = 0;
}

/* no LEDs available on my lpc2106 board... :-( */
#define led_init()			{}
#define led_indicate_error()		{}
#define led_indicate_running()		{}
#define led_indicate_idle()		{}
#define led_indicate_kernelinit()	{}

#endif
