/* Copyright (c) 2009, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#include "asm.h"
#include "cpu.h"

.text
.arm

.global _start
_start:
	/* exception vector */
	ldr	pc, =bootstrap		/* 0x00  svc  reset */
	ldr	pc, =isr_undefined	/* 0x04  und  undefined instructions */
	ldr	pc, =isr_syscall	/* 0x08  svc  software interrupt */
	ldr	pc, =isr_prefetch	/* 0x0c  abt  prefetch abort */
	ldr	pc, =isr_data		/* 0x10  abt  data abort */
	nop				/* 0x14 reserved for future expansion */
	ldr	pc, [pc, #-0xff0]	/* 0x18  irq  interrupt (VIC-handled) */
	ldr	pc, =unexpected		/* 0x1c  fiq  fast interrupt */

bootstrap:
	/* setup phase locked loop */
	ldr	r0,	=0xe01fc080	/* base address */
	mov	r1,	#(PLL_MUL - 1)	/* set multiplier */
	str	r1,	[r0, #0x4]
	mov	r1,	#1		/* enable PLL */
	str	r1,	[r0, #0x0]
	mov	r2,	#0xaa		/* send feed seq to PLL */
	mov	r3,	#0x55
	str	r2,	[r0, #0xc]
	str	r3,	[r0, #0xc]
1:	ldr	r1,	[r0, #0x8]	/* wait for pll till it is locked */
	ands	r1, r1, #(1 << 10)
	beq	1b
	mov	r1,	#3		/* enable and connect PLL clock */
	str	r1,	[r0, #0x0]
	str	r2,	[r0, #0xc]	/* send feed again */
	str	r3,	[r0, #0xc]

	/* setup memory accelerator module */
	ldr	r0,	=0xe01fc000	/* base address */
	mov	r1,	#(MAM_CCLKS)
	str	r1,	[r0, #0x4]
	mov	r1,	#2		/* fully enable MAM */
	str	r1,	[r0, #0x0]

	/* set vpbdiv */
	ldr	r0,	=0xe01fc100
	mov	r1,	#1
	str	r1,	[r0, #0x0]

	/* configure pin 0.0 and 0.1 as UART pins, and pin 0.4-0.7 as SPI
	 * pins
	 * TODO: move this to be more flexible */
	ldr	r0,	=0xe002c000
	ldr	r1,	=0x00005505
	str	r1,	[r0, #0x0]

	init_bss

	b	init
