/* Copyright (c) 2009, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once

#define PLATFORM_NAME	CONFIG_CPU_VARIANT " with " CONFIG_SDRAM " sdram chip"

/* pin configuration */
#define PINSEL0		0x00000050	/* UART RXD0, TXD0*/
#define PINMODE0	0x000000a0
#define PINSEL1		0x00000000
#define PINMODE1	0x00000000
#ifdef CONFIG_LPCMCI
# define PINSEL2	0x0280a8a0	/* MCI MCICLK, MCICMD, MCIPWR, MCIDAT[0:3] */
# define PINMODE2	0x00000000
#else
# define PINSEL2	0x00000000
# define PINMODE2	0x00000000
#endif
#define PINSEL3		0x00000000
#define PINMODE3	0x00000000
#define PINSEL4		0x50000000	/* RAM CS[23] */
#define PINMODE4	0xa0000000
#define PINSEL5		0x55015555	/* RAM CAS, RAS, CLKOUT[01], DYCS[0:3], CKEOUT0, DQMOUT[0:3] */
#define PINMODE5	0xaa02aaaa
#define PINSEL6		0x55555555	/* RAM D[0:15] */
#define PINMODE6	0xaaaaaaaa
#define PINSEL7		0x55555555	/* RAM D[16:31] */
#define PINMODE7	0xaaaaaaaa
#define PINSEL8		0x55555555	/* RAM A[0:15] */
#define PINMODE8	0xaaaaaaaa
#define PINSEL9		0x55555554	/* RAM A[17:23], OE, WE, BLS[0:3], CS[01] */
#define PINMODE9	0xaaaaaaa8

#define OSCIFREQ	12000000
#define PLLMUL		24
#define	CPUDIV		4
#define	USBDIV		6
#define CPU_CLOCK	(OSCIFREQ * PLLMUL / CPUDIV)	/* 72 MHz */

/* clocks for timing of MAM
 * LPC24xx User Manual suggests the following timings:
 * cpu clock	cycles
 * < 20 MHz	1
 * 20 - 40 MHz	2
 * 40 - 60 MHz	3
 * > 60 MHz	4
 * Thus the following formula is a good approximation */
#define MAMTIM	(CPU_CLOCK / 20000000 + 1)

/* this is internal RAM. */
#define RAM_START	0x40000000
#define RAM_SIZE	(64 << 10)

#ifndef CONFIG_SDRAM_NONE
#define HAS_EXTERNAL_RAM
#define EXTRAM_START	0xa0000000
#endif

/* Samsung 512K x 32Bit x 4 Banks Synchronous DRAM */
#ifdef CONFIG_SDRAM_K4S643232E
#include "ram_k4s643232e.h"
#endif

/* Micron 128 Mb SDRAM */
#ifdef CONFIG_SDRAM_MT48LC4M32B2
#include "ram_mt48lc4m32b2.h"
#endif

/* base address of uart0 used as console */
#define UART0_BASE	0xe000c000

/* register controlling power of peripherials */
#define PCOMP_REG	(*(volatile int *) 0xe01fc0c4)

/* base of timer0 used for scheduling */
#define TIMER0_BASE	0xe0004000

/* VIC registers */
#define VICIntEnable	(* (volatile int  *)  0xfffff010)
#define VICcVctAddr	(* (volatile long *)  0xffffff00)
#define VICVectAddr(x)	(* (volatile long *) (0xfffff100 + x*4))

#define TIMER_IRQ	4	/* timer interrupt number */
#define UART_IRQ	6	/* uart0 interrupt number */

/* TODO: generic GPIO driver */
#define GPIO_LED_BASE	0x3fffc000
#define GPIO_LED_DIR	(* (volatile int *) (GPIO_LED_BASE + 0x40))
#define GPIO_LED_SET	(* (volatile int *) (GPIO_LED_BASE + 0x58))
#define GPIO_LED_CLEAR	(* (volatile int *) (GPIO_LED_BASE + 0x5c))

#define LED_RUNNING	(1 << 25)	/* blue LED */
#define LED_KERNELINIT	(1 << 26)	/* green LED */
#define LED_ERROR	(1 << 27)	/* red LED */

#define cpu_idle_asm						\
	ldr	r0,	=0xe01fc0c0;	/* PCON register */	\
	mov	r1,	#(1 << 0);	/* Idle mode */		\
	str	r1,	[r0];

#ifndef __ASSEMBLER__

/* sets a function for an interrupt as ISR */
static inline void vic_set(int nr, void (*isr)(void))
{
	VICIntEnable |= (1 << nr);

	VICVectAddr(nr) = (long) isr;
}

/* resets VIC. Should be called at the end of every ISR */
static inline void vic_reset(void)
{
	VICcVctAddr = 0;
}

/* functions for LED to indicate system status */

static inline void led_init(void)
{
	/* set pin direction as output pins */
	GPIO_LED_DIR = (LED_RUNNING | LED_KERNELINIT | LED_ERROR);

	/* set pins to low except of kernelinit (set to low when kernel is
	 * initalized) */
	GPIO_LED_CLEAR = (LED_RUNNING | LED_ERROR);
	GPIO_LED_SET = LED_KERNELINIT;
}

static inline void led_indicate_error(void)
{
	GPIO_LED_SET = LED_ERROR;
}

static inline void led_indicate_running(void)
{
	GPIO_LED_SET = LED_RUNNING;
}

static inline void led_indicate_idle(void)
{
	GPIO_LED_CLEAR = LED_RUNNING;
}

static inline void led_indicate_kernelinit(void)
{
	GPIO_LED_CLEAR = LED_KERNELINIT;
}

#endif
