/* Copyright (c) 2009, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#include "asm.h"
#include "cpu.h"

.text
.arm

.global _start
_start:
	/* exception vector */
	ldr	pc, =bootstrap		/* 0x00  svc  reset */
	ldr	pc, =isr_undefined	/* 0x04  und  undefined instructions */
	ldr	pc, =isr_syscall	/* 0x08  svc  software interrupt */
	ldr	pc, =isr_prefetch	/* 0x0c  abt  prefetch abort */
	ldr	pc, =isr_data		/* 0x10  abt  data abort */
	nop				/* 0x14 reserved for future expansion */
	ldr	pc, [pc, #-0x120]	/* 0x18  irq  interrupt (VIC-handled) */
	ldr	pc, =unexpected		/* 0x1c  fiq  fast interrupt */

bootstrap:
	ldr	r0,	=0xe01fc000	/* base of some system control regs */

	/* enable main oscillator */
	ldr	r1,	=(1 << 5)	/* main oscillator enable */
	str	r1,	[r0, #0x1a0]
1:	ldr	r1,	[r0, #0x1a0]	/* wait for oscillator to be ready */
	ands	r1, 	r1,	#(1 << 6)
	beq	1b

	/* initalize pll */
	ldr	r2,	=0xaa		/* feed sequence */
	ldr	r3,	=0x55
	ldr	r1,	=0		/* disable PLL */
	str	r1,	[r0, #0x080]
	str	r2,	[r0, #0x08c]
	str	r3,	[r0, #0x08c]
	ldr	r1,	=1		/* use main osci as PLL source */
	str	r1,	[r0, #0x10c]
	ldr	r1,	=((PLLMUL-1) | 1 << 16)	/* M: PLLMUL, D: 1 */
	str	r1,	[r0, #0x084]
	str	r2,	[r0, #0x08c]
	str	r3,	[r0, #0x08c]
	ldr	r1,	=1		/* enable */
	str	r1,	[r0, #0x080]
	str	r2,	[r0, #0x08c]
	str	r3,	[r0, #0x08c]
	ldr	r1,	=(CPUDIV-1)	/* CCLKCFG */
	str	r1,	[r0, #0x104]
	ldr	r1,	=(USBDIV-1)	/* USBCLKCFG */
	str	r1,	[r0, #0x108]
1:	ldr	r1,	[r0, #0x088]	/* wait for pll to be locked */
	ands	r1,	r1,	#(1 << 26)
	beq	1b
	ldr	r1,	=3		/* enable & connect pll */
	str	r1,	[r0, #0x080]
	str	r2,	[r0, #0x08c]
	str	r3,	[r0, #0x08c]

	/* set CCLK/1 as peripherial clock */
	ldr	r1,	=0x55515555
	str	r1,	[r0, #0x1a8]
	ldr	r1,	=0x11555455
	str	r1,	[r0, #0x1ac]

	/* initalize Memory Accelerator Module */
	ldr	r1,	=MAMTIM
	str	r1,	[r0, #0x004]
	ldr	r1,	=2		/* fully enable */
	str	r1,	[r0, #0x000]

	/* enable high speed GPIO at ports 0 and 1 */
	ldr	r1,	=(1 | 1 << 5)
	str	r1,	[r0, #0x1a0]

	/* configure PIN functions and modes */
	ldr	r0,	=0xe002c000
#define PIN(x)				\
	ldr	r1,	=PINSEL##x;	\
	str	r1,	[r0, #(x*4)];	\
	ldr	r1,	=PINMODE##x;	\
	str	r1,	[r0, #(0x40+x*4)]
	PIN(0)
	PIN(1)
	PIN(2)
	PIN(3)
	PIN(4)
	PIN(5)
	PIN(6)
	PIN(7)
	PIN(8)
	PIN(9)

	/* initalize external SDRAM */
	ldr	r0,	=0xffe08000	/* EMC base */
	ldr	r1,	=1
	str	r1,	[r0, #0x000]	/* EMC enable */
	ldr	r1,	=R_RS
	str	r1,	[r0, #0x028]	/* set read strategy */
	ldr	r1,	=R_RASCAS
	str	r1,	[r0, #0x104]	/* RAS & CAS timings */
	ldr	r1,	=R_tRP
	str	r1,	[r0, #0x030]	/* tRP timing */
	ldr	r1,	=R_tRAS
	str	r1,	[r0, #0x034]	/* tRAS timing */
	ldr	r1,	=R_tSREX
	str	r1,	[r0, #0x038]	/* tSREX timing */
	ldr	r1,	=R_tAPR
	str	r1,	[r0, #0x03c]	/* tAPR timing */
	ldr	r1,	=R_tDAL
	str	r1,	[r0, #0x040]	/* tDAL timing */
	ldr	r1,	=R_tWR
	str	r1,	[r0, #0x044]	/* tWR timing */
	ldr	r1,	=R_tRC
	str	r1,	[r0, #0x048]	/* tRC timing */
	ldr	r1,	=R_tRFC
	str	r1,	[r0, #0x04c]	/* tRFC timing */
	ldr	r1,	=R_tXSR
	str	r1,	[r0, #0x050]	/* tXSR timing */
	ldr	r1,	=R_tRRD
	str	r1,	[r0, #0x054]	/* tRRD timing */
	ldr	r1,	=R_tMRD
	str	r1,	[r0, #0x058]	/* tMRD timing */
	ldr	r1,	=R_ADRMAP
	str	r1,	[r0, #0x100]	/* address mappings */
	/* init sequence */
	ldr	r1,	=0
1:	add	r1,	r1,	#1
	cmp	r1,	#128
	bne	1b
	ldr	r1,	=0x183
	str	r1,	[r0, #0x020]
	ldr	r1,	=0
1:	add	r1,	r1,	#1
	cmp	r1,	#128
	bne	1b
	ldr	r1,	=0x101
	str	r1,	[r0, #0x020]
	ldr	r1,	=1
	str	r1,	[r0, #0x024]
	ldr	r1,	=0
1:	add	r1,	r1,	#1
	cmp	r1,	#128
	bne	1b
	ldr	r1,	=19
	str	r1,	[r0, #0x024]
	ldr	r1,	=0x081
	str	r1,	[r0, #0x020]
	ldr	r1,	=(0xa0000000 | (0x32 << 12))
	ldr	r2,	[r1]
	ldr	r1,	=0x000
	str	r1,	[r0, #0x020]
	ldr	r1,	[r0, #0x100]
	orr	r1,	r1,	#(1 << 19)
	str	r1,	[r0, #0x100]

	init_bss

	b	init
