/*-
 * Copyright (c) 2010 Alexander Graf.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Registers */
#define WDBASE	0xe0000000
#define WDMOD	(*(volatile int *) (WDBASE+0x0))	/* Mode	*/
#define WDTC	(*(volatile int *) (WDBASE+0x4))	/* Timer constant */
#define WDFEED	(*(volatile int *) (WDBASE+0x8))	/* Feed sequence */

/* Mode flags */
#define WDEN	(1 << 0)	/* Enable */
#define WDRESET	(1 << 1)	/* Reset enable (on underflow) */
#define WDTOF	(1 << 2)	/* Time-out flag */

/* Feed */
#define WDFD1	0xaa
#define WDFD2	0x55

/*
 * The Timer Constant is the time in microseconds (4 MHz oscillator divided by 4
 * decrements a counter set up with TC (on feed).  If it reaches 0, the WDT
 * issues a reset.)
 *
 * As we feed on every scheduling tick issued by the timer, it should be a bit
 * higher (we use *1.5) than the scheduling interval.
 */
#define TCVAL	((3 * 1000000 / 2) / SCHED_HZ)

static inline void __optimize wdt_feed(void)
{
	WDFEED = WDFD1;
	WDFEED = WDFD2;
}

static inline void wdt_init(void)
{
	if (WDMOD & WDTOF)
		puts("WARNING: Last reset was issued by Watchdog Timer.");

	/* Enable the Watchdog Timer and tell the dog that it should bite us if
	 * we forget to feed him. */
	WDMOD = WDEN | WDRESET;

	WDTC = TCVAL;

	/* Feeding is needed to activate the WDT */
	wdt_feed();
}
