/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once


/*
 * needed in userspace sys/wait.h
 */


/* options for waitpid() */
#define WCONTINUED	(1 << 0)
#define WNOHANG		(1 << 1)
#define WUNTRACED	(1 << 2)


/*
 * needed in userspace poll.h
 */


/* events for poll() */
#define POLLIN		(POLLRDNORM | POLLRDBAND)
#define POLLRDNORM	(1 << 0)
#define POLLRDBAND	(1 << 1)
#define POLLPRI		(1 << 2)
#define POLLOUT		(1 << 3)
#define POLLWRNORM	POLLOUT
#define POLLWRBAND	(1 << 4)
#define POLLERR		(1 << 5)
#define POLLHUP		(1 << 6)
#define POLLNVAL	(1 << 7)

struct pollfd {
	int fd;
	short events;
	short revents;
};


/*
 * needed in userspace signal.h
 */


/* signals. The numbers are, where possible, aligned with the POSIX definition
 * of the /bin/kill utility. Where the number is not defined, it is sorted by
 * alphabet. */
#define SIGHUP		1
#define SIGINT		2
#define SIGQUIT		3
#define SIGBUS		4
#define SIGCHLD		5
#define SIGABRT		6
#define SIGCONT		7
#define SIGFPE		8
#define SIGKILL		9
#define SIGILL		10
#define SIGPIPE		11
#define SIGPOLL		12	/* in POSIX marked with OB XSR */
#define SIGPROF		13	/* in POSIX marked with OB XSR */
#define SIGALRM		14
#define SIGTERM		15
#define SIGSEGV		16
#define SIGSTOP		17
#define SIGSYS		18	/* in POSIX marked with XSI */
#define SIGTRAP		19
#define SIGTSTP		20
#define SIGTTIN		21
#define SIGTTOU		22
#define SIGURG		23
#define SIGUSR1		24
#define SIGUSR2		25
#define SIGUSR3		26	/* not defined in POSIX */
#define SIGUSR4		27	/* not defined in POSIX */
#define SIGUSR5		28	/* not defined in POSIX */
#define SIGVTALRM	29	/* in POSIX marked with XSI */
#define SIGXCPU		30
#define SIGXFSZ		31

/* number of signals */
#define SIGNUM		32

/* action parameters */
#define SIG_ERR		((void *) -1)
#define SIG_DFL		((void *) 1)
#define SIG_IGN		((void *) 2)

/* flags for sigaction */
#define SA_NOCLDSTOP	(1 << 0)
#define SA_ONSTACK	(1 << 1)
#define SA_RESETHAND	(1 << 2)
#define SA_RESTART	(1 << 3)
#define SA_SIGINFO	(1 << 4)
#define SA_NOCLDWAIT	(1 << 5)
#define SA_NODEFER	(1 << 6)

/* sigaction structure for sigaction() */
struct sigaction {
	void (*sa_handler)(int);
	int sa_flags;
	/* TODO: have sa_mask and sa_sigaction */
};


/*
 * needed in userspace sched.h
 */


/* info struct for sched_info() */
struct sched_info {
	int hz;		/* frequency */
	int procs;	/* maximum size of process list */
	int memtotal;	/* total amount of memory (pages) */

	int la1;	/* 1000 * loadavg, last minute */
	int la5;	/* 1000 * loadavg, last five minutes */
	int la15;	/* 1000 * loadavg, last fifteen minutes */

	unsigned int uptime;	/* system uptime in milliseconds */
};

/* info struct for sched_list() */
struct sched_procinfo {
	int state;	/* one of the STATE_ below */
	int pid;	/* process id */
	int ppid;	/* parent process id */
	int memsz;	/* used memory pages */
	char *cmd;	/* command line */
	unsigned int cputime; /* cpu time */
};

/* state values */
#define STATE_ZOMBIE	'Z'
#define STATE_RUNNING	'R'
#define STATE_SLEEPING	'S'
#define STATE_STOPPED	'T'

#ifdef KERNEL


/*
 * needed only kernel-internally
 */


#include "compiler.h"

/* task switch frequency */
#define SCHED_HZ	CONFIG_SCHED_HZ

/* defines concerning load average computing */
#define SCHED_LA_PERMIN	CONFIG_LAC_PERMIN	/* loadavg updates per minute */
#define SCHED_LA_TICKS	(60 / SCHED_LA_PERMIN * SCHED_HZ) /* each ... ticks */
#define SCHED_LA_TOTCLC	(15 * SCHED_LA_PERMIN) /* calculations needed to keep */

/* maximum number of processes */
#define SCHED_MAX_PROC	CONFIG_MAX_PROC

/* maximum pid number */
#define SCHED_MAX_PID	CONFIG_MAX_PID

/* stack size for processes (in pages) */
#define SCHED_PROC_SP	(CONFIG_PROC_STACKSIZE / PAGE_SIZE)

/* process states */
#define ST_ZOMBIE	1
#define ST_RUNNING	2
#define ST_RUNNING_NEXT	3
#define ST_SLEEPING	4
#define ST_STOPPED	5

/* wt_func arguments for block / wake up functions */
#define WT_WAITPID	1
#define WT_READ		2
#define WT_WRITE	3
#define WT_POLL		4

struct context {
	long psr;
	long r0, r1, r2, r3, r4, r5, r6, r7;
	long r8, r9, sl, fp, ip, sp, lr;
	long pc;
};

/* linked list with information for blocking functions */
struct iowait_info {
	/* next in list (if poll) */
	struct iowait_info *next;

	/* awaited event */
	int event;

	/* filedescriptor which blocks */
	struct filedes *fd;
};

/* information struct needed when being in an signal to restore old state
 * afterwards
 * TODO: support signal queue
 * TODO: also keep polling information, etc. */
struct sigctx {
	struct context ctx;
	int state;
	int signum;
};

/* defined in elf.c */
struct elf_sym;

/* information struct about each process */
struct process {
	/* current state of the process, one of the ST_ constants above */
	int state;

	/* additional info about the current state. */
	union {
		struct {
			/* exit status of zombie processes */
			int exit_status;
		} zombie;

		struct {
			/* function which blocks the process */
			int wt_func;

			/* information for wt_func = WT_READ, WT_WRITE or
			 * WT_POLL */
			struct iowait_info *iowait_info;

			/* struct pollfd *fds for poll which has to be modified
			 * before returning */
			struct pollfd *pollfd;

			/* value of timer_ticks when poll timed out, or 0 if it
			 * has no timeout set */
			unsigned int timeout_tick;
		} sleeping;
	} sti;

	/* process id and parent process id of the process. It is not the same
	 * as the tid (task id) which is the index in the procs array of the
	 * process. */
	int pid, ppid;

	/* start of the stack of the process */
	void *stack;

#ifdef CONFIG_SYMTAB
	/* ELF offset in RAM */
	void *elfoff;

	/* ELF symbol string table */
	char *elfsymstrtab;

	/* ELF symbol table */
	struct elf_sym *elfsymtab;

	/* ELF symbol count */
	int elfsymnum;
#endif

	/* linked list containing all of the process' open file descriptors */
	struct filedes *fds;

	/* command line the process was invoked with */
	char *cmdline;

	/* set to 0, after fork(), set to 1 after execve(). */
	int img_replaced;

	/* CPU time spent in this process */
	unsigned int cputime;

	/* uptime when the process became the running process */
	unsigned int got_running;

	/* configured signal handlers */
	struct sigaction sa[SIGNUM];

	/* former context. This is 0 if the process is not handling a signal and
	 * if the process is handling a signal, this points to the context at
	 * time of interruption */
	struct sigctx *sigctx;

	/* old state in case of stopped process */
	int old_state;

	/* registers of the process when it was interrupted the last time */
	struct context ctx;
};

extern struct process *procs[SCHED_MAX_PROC];

/* task id (index in procs) of the current running process */
extern int current;

extern void sched_init(void);

extern void sched(struct context *ctx, int from_timer);
extern int sys_fork(struct context *);
extern void sys__exit(struct context *ctx, int tid, int status);
extern int sys_waitpid(struct context *, int, int *, int);

/* functions for sleep / wake up control */
struct io_handle;	/* defined in bio.h */
extern int proc_block_io(struct context *ctx, int tid, int wt_func, int event,
			 struct filedes *fd);
extern void proc_wake_io(struct context *ctx, int event,
			 const struct io_handle *ioh);

extern int sys_poll(struct context *ctx, struct pollfd *fds, unsigned int nfds,
		    int timeout);

/* reset signal handlers to default signal handlers, needed for execve() and
 * initial process creation */
extern void sig_reset_handlers(int tid);
/* send signal to the given task */
extern void sig_emit(struct context *ctx, int tid, int sig);
/* wrapper around sig_emit(), secure enough for userspace */
extern int sys_kill(int pid, int sig);
extern void sys_sigrestorectx(void);
extern int sys_sigaction(int sig, const struct sigaction *act,
			 struct sigaction *oact);

/* gives sched_info struct, returns 0 */
extern int sys_sched_info(struct sched_info *si);
/* writes process table of struct sched_procinfo with up to n entries. Returns
 * the number of valid entries in the table */
extern int sys_sched_list(struct sched_procinfo *spi, int n);

/* in main.c, entry point for first process */
extern __noreturn init2(void);

/* in fault.c, is called by the scheduler for some signals to dump GPRs. */
extern void exception(struct context *ctx, int tid, int sig);

#endif
