/* Copyright (c) 2009-2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

#pragma once

/* this file contains a list of syscall numbers. for information about what's
 * done with the registers, look at syscall.c. Normally r0 contains the return
 * value (>= 0 on success, negative errno on failure). The registers starting
 * with r0 contain function parameters. */

/* the syscall number is stored in the 24 bit immediate field of the SWI
 * instruction. The lowest SC_C_S bits are used for the function number, all
 * higher bits for the function group number. */

#define SC_C_S	3

/* syscall groups */
#define SC_C_SCHED	(0 << SC_C_S)	/* process control functions */
#define SC_C_SIG	(1 << SC_C_S)	/* signal functions */
#define SC_C_PROCCFG	(2 << SC_C_S)	/* process info / config functions */
#define SC_C_MM		(3 << SC_C_S)	/* memory management functions */
#define SC_C_IO		(4 << SC_C_S)	/* input/output functions */
#define SC_C_SYSCFG	(5 << SC_C_S)	/* system info / config functions */

/* functions */

#define SC_SCHED_YIELD	(SC_C_SCHED | 0)
#define SC_FORK		(SC_C_SCHED | 1)
#define SC__EXIT	(SC_C_SCHED | 2)
#define SC_EXECVE	(SC_C_SCHED | 3)
#define SC_WAITPID	(SC_C_SCHED | 4)
#define SC_POLL		(SC_C_SCHED | 5)

#define SC_KILL		(SC_C_SIG | 0)
#define SC_SIGRESTORECTX (SC_C_SIG | 1)
#define SC_SIGACTION	(SC_C_SIG | 2)

#define SC_GETPID	(SC_C_PROCCFG | 0)
#define SC_GETPPID	(SC_C_PROCCFG | 1)
#define SC_SETSID	(SC_C_PROCCFG | 2)

#define SC_MMAP		(SC_C_MM | 0)
#define SC_MUNMAP	(SC_C_MM | 1)

#define SC_OPEN		(SC_C_IO | 0)
#define SC_CLOSE	(SC_C_IO | 1)
#define SC_READ		(SC_C_IO | 2)
#define SC_WRITE	(SC_C_IO | 3)

#define SC_SCHED_INFO	(SC_C_SYSCFG | 0)
#define SC_SCHED_LIST	(SC_C_SYSCFG | 1)

#ifndef __ASSEMBLER__
#ifdef KERNEL

struct context;

/* is the context in which a syscall is made. Only valid in functions called by
 * syscall(), as points to the stack of the syscall() function */
extern struct context *sc_ctx;

#endif
#endif
