/* Copyright (c) 2010, Alexander Graf <alex@wortkreuzung.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. */

struct context;		/* defined in sched.h */
struct process;		/* defined in sched.h */
struct term_info;	/* defined in terminal.c */

/* shall be called by hardware driver to initalize terminal
 *
 * ttyname: name of the tty relative to /dev/
 * putchar: function of the driver which puts one char */
extern struct term_info *term_init(const char *ttyname, void (*putchar)(char));

/* should be called by hardware drier whenever a new character arrives */
extern void term_newc(struct context *ctx, struct term_info *ti, char c);

extern int sys_setsid(struct context *ctx);

/* to be called by fork() to add the new process to the group in which the
 * parent process is (if any) */
extern void term_on_fork(struct process *old, struct process *new, int newtid);

/* to be called by _exit() to remove the exitted process from its process group
 * (if any) */
extern void term_on_exit(struct context *ctx, struct process *proc);
